C (C) Copyright 2005- ECMWF.
C (C) Copyright 2013- Meteo-France.
C 
C This software is licensed under the terms of the Apache Licence Version 2.0
C which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
C In applying this licence, ECMWF does not waive the privileges and immunities
C granted to it by virtue of its status as an intergovernmental organisation
C nor does it submit to any jurisdiction.
C

      subroutine mpi_irecv
      call abort()
      return
      end
